
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include "dwt.h"


void usage(char *s)
{
	if (s) 
		printf("dwt: %s\n\n", s);
	else 
		printf("Davis Weather Tool - dwt 1.0\n\n");

	printf("usage: dwt COMMAND PARAMS [options]\n\
\n\
   COMMANDS      PARAMS\n\
        FRMT    +FORMAT    Output formatted observation fields as defined in format string\n\
        TAGS               List observation tags used for format string\n\
        LIST               List all observation fields\n\
        GETTIME            Display time of receiver\n\
        SETTIME            Set time of receiver to that of PC\n\
        TEST               Test communications\n\
        WRD                Display weather station type\n\
        VER                Display firmware version\n\
        RXCHECK            Display receiver statistics\n\
     \n\
 Options:\n\
       -s tty     Specify alternate serial port\n\
       -r num     Repeat FRMT command 'num' times\n\
       -d delay   Delay after FRMT command, valid if -r option used\n\n");

	exit(-1);
}



int main(int argc, char **argv)
{
	int	fd, c;
	ushort	cmd, num = 1, delay = 0;
	char	*tty = NULL, *frmt = NULL;
	char *buff = NULL;
	struct tm *tp = NULL;
	int type;

	if (argc < 2) 
		usage(NULL);

	if (!valid_cmd(argv[1]))
		usage("Invalid command");

	cmd = get_cmd(argv[1]);

	if (cmd == 1) {
		if (argc < 3)
			usage(NULL);

		if  (argv[2][0] != '+')
			usage("Invalid format specifier");

		frmt = argv[2]+1;
	}

	opterr=0;

	while ( (c = getopt( argc, argv, "s:r:d:")) != EOF)
		switch(c)
                {
                        case 's' :    
				tty = strdup(optarg);
                                break;
                        case 'r' :
				num = atoi(optarg);
                                break;
			case 'd'
				:
				delay = atoi(optarg);
				break;
                        default :
                                opterr++;
                                break;                                
                }                                                     

	if (opterr) 
		usage("Invalid options");

	if ((num>1  || delay>0) && (cmd != 1))
		usage("Invalid options - only valid for FRMT command");

	if (tty == NULL) 
		tty = DWT_PORT;

	if ((fd = open_dwt(tty)) == -1) {
		fprintf(stderr,"dwt: Cannot open serial port");
		exit(-1);
	}

	if (cmd  != 2)
		wake_recv(fd);

	switch(cmd) {
	case 1 :	
		CMD_frmt(fd, frmt, num, delay);
		break;
		
	case 2 :	CMD_tags();
		
		break;
		
	case 3 :
		CMD_list(fd);
		break;
	case 4 :
		tp = CMD_gettime(fd);
		if (tp != NULL) {
			printf("Receiver time currently - %02d:%02d:%02d %d-%d-%d\n",
			       tp->tm_hour, tp->tm_min, tp->tm_sec, 
			       tp->tm_mday, tp->tm_mon+1, tp->tm_year+1900);
			free(tp);
		}
		break;
	case 5 :
		CMD_settime(fd);
		break;
	case 6 :
		buff = CMD_test(fd);
		if (buff != NULL) {
			printf("%s\n", buff);
			free(buff);
		}
		break;
	case 7 :
		if ((type = CMD_wrd(fd)) < 0) {
			printf("wrd: failed\n");
		} else {
			printf("Davis Weather Station Type: ");
			buff = dwt_type(type);
			printf("%s\n", buff);
			free(buff);
		}
		break;
	case 8 :
		buff = CMD_ver(fd);
		if (buff != NULL) {
			printf("%s\n", buff);
			free(buff);
		}
		break;
	case 9 :
		buff = CMD_rxcheck(fd);
		if (buff != NULL) {
			printf("%s\n", buff);
			free(buff);
		}
		break;
	default :
		break;
	}
	close_dwt(fd);
	exit(0);
}
